@@ -43,6 +43,8 @@
 #' @param id The name of the id variable in the input data.
 #' @param ties The method for handling ties, either "breslow" or 
 #'   "efron" (default).
+#' @param init The vector of initial values. Defaults to zero for all 
+#'   variables.
 #' @param robust Whether a robust sandwich variance estimate should be
 #'   computed. In the presence of the id variable, the score residuals
 #'   will be aggregated for each id when computing the robust sandwich
@@ -83,6 +85,8 @@
 #'     - \code{robust}: Whether to use the robust variance estimate.
 #'
 #'     - \code{firth}: Whether to use Firth's penalized likelihood method.
+#'     
+#'     - \code{fail}: Whether the model fails to converge.
 #'
 #'     - \code{loglik0_unpenalized}: The unpenalized log-likelihood under null.
 #'
@@ -227,7 +231,8 @@
 phregr <- function(data, rep = "", stratum = "",
                    time = "time", time2 = "", event = "event",
                    covariates = "", weight = "", offset = "",
-                   id = "", ties = "efron", robust = FALSE,
+                   id = "", ties = "efron", 
+                   init = NA_real_,  robust = FALSE,
                    est_basehaz = TRUE, est_resid = TRUE, firth = FALSE,
                    plci = FALSE, alpha = 0.05, 
                    maxiter = 50, eps = 1.0e-9) {
@@ -236,27 +241,27 @@ phregr <- function(data, rep = "", stratum = "",
   
   elements = c(rep, stratum, time, event, covariates, weight, offset, id)
   elements = unique(elements[elements != "" & elements != "none"])
-  mf = model.frame(formula(paste("~", paste(elements, collapse = "+"))),
-                   data = data)
+  fml = formula(paste("~", paste(elements, collapse = "+")))
+  mf = model.frame(fml, data = data, na.action = na.omit)
+  
   rownum = as.integer(rownames(mf))
   df = data[rownum,]
   
   nvar = length(covariates)
   if (missing(covariates) || is.null(covariates) || (nvar == 1 && (
     covariates[1] == "" || tolower(covariates[1]) == "none"))) {
-    t1 = terms(formula("~1"))
     p = 0
+    t1 = terms(formula("~1"))
   } else {
-    t1 = terms(formula(paste("~", paste(covariates, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
-    p = length(t3)
+    fml1 = formula(paste("~", paste(covariates, collapse = "+")))
+    p = length(rownames(attr(terms(fml1), "factors")))
+    t1 = terms(fml1)
   }
   
   if (p >= 1) {
-    mf = model.frame(t1, df)
-    xlevels = mf$xlev
-    mm = model.matrix(t1, mf)
+    mf1 <- model.frame(fml1, data = df, na.action = na.pass)
+    mm <- model.matrix(fml1, mf1)
+    xlevels = mf1$xlev
     param = colnames(mm)
     colnames(mm) = make.names(colnames(mm))
     varnames = colnames(mm)[-1]
@@ -273,7 +278,8 @@ phregr <- function(data, rep = "", stratum = "",
   
   fit <- phregcpp(data = df, rep = rep, stratum = stratum, time = time,
                   time2 = time2, event = event, covariates = varnames,
-                  weight = weight, offset = offset, id = id, ties = ties,
+                  weight = weight, offset = offset, id = id, 
+                  ties = ties, init = init, 
                   robust = robust, est_basehaz = est_basehaz,
                   est_resid = est_resid, firth = firth,
                   plci = plci, alpha = alpha, 
