@@ -30,7 +30,7 @@ List ipcwcpp(
     const bool swtrt_control_only = 1,
     const double alpha = 0.05,
     const std::string ties = "efron",
-    const bool boot = 1,
+    const bool boot = 0,
     const int n_boot = 1000,
     const int seed = NA_INTEGER) {
   
@@ -262,8 +262,11 @@ List ipcwcpp(
   NumericVector swtrt_timenz = data[swtrt_time];
   NumericVector swtrt_timen = clone(swtrt_timenz);
   for (i=0; i<n; i++) {
+    if (swtrtn[i] == 1 && std::isnan(swtrt_timen[i])) {
+      stop("swtrt_time must not be missing when swtrt=1");
+    }
     if (swtrtn[i] == 1 && swtrt_timen[i] < 0.0) {
-      stop("swtrt_time must be nonnegative");
+      stop("swtrt_time must be nonnegative when swtrt=1");
     }
   }
   
@@ -403,6 +406,36 @@ List ipcwcpp(
   }
   
   
+  // exclude observations with missing values
+  LogicalVector sub(n,1);
+  for (i=0; i<n; i++) {
+    if ((idn[i] == NA_INTEGER) || (stratumn[i] == NA_INTEGER) || 
+        (std::isnan(tstartn[i])) || (std::isnan(tstopn[i])) || 
+        (eventn[i] == NA_INTEGER) || (treatn[i] == NA_INTEGER) || 
+        (swtrtn[i] == NA_INTEGER)) {
+      sub[i] = 0;
+    }
+    for (j=0; j<q+p2; j++) {
+      if (std::isnan(zn_lgs_den(i,j))) sub[i] = 0;
+    }
+  }
+  
+  IntegerVector order = which(sub);
+  idn = idn[order];
+  stratumn = stratumn[order];
+  tstartn = tstartn[order];
+  tstopn = tstopn[order];
+  eventn = eventn[order];
+  treatn = treatn[order];
+  swtrtn = swtrtn[order];
+  swtrt_timen = swtrt_timen[order];
+  if (p > 0) zn = subset_matrix_by_row(zn, order);
+  zn_cox_den = subset_matrix_by_row(zn_cox_den, order);
+  zn_lgs_den = subset_matrix_by_row(zn_lgs_den, order);
+  n = sum(sub);
+  if (n == 0) stop("no observations left after removing missing values");
+  
+  
   // split at treatment switching into two observations if treatment 
   // switching occurs strictly between tstart and tstop for a subject
   LogicalVector tosplit(n);
@@ -452,25 +485,14 @@ List ipcwcpp(
   }
   
   
-  // sort data by treatment group, stratum, id, and time
-  IntegerVector order = seq(0, n-1);
-  if (has_stratum) {
-    std::sort(order.begin(), order.end(), [&](int i, int j) {
-      return ((treatn[i] < treatn[j]) ||
-              ((treatn[i] == treatn[j]) && (stratumn[i] < stratumn[j])) ||
-              ((treatn[i] == treatn[j]) && (stratumn[i] == stratumn[j]) &&
-              (idn[i] < idn[j])) ||
-              ((treatn[i] == treatn[j]) && (stratumn[i] == stratumn[j]) &&
-              (idn[i] == idn[j]) && (tstopn[i] < tstopn[j])));
-    });
-  } else {
-    std::sort(order.begin(), order.end(), [&](int i, int j) {
-      return ((treatn[i] < treatn[j]) ||
-              ((treatn[i] == treatn[j]) && (idn[i] < idn[j])) ||
-              ((treatn[i] == treatn[j]) && (idn[i] == idn[j]) && 
-              (tstopn[i] < tstopn[j])));
-    });
-  }
+  // sort data by treatment group, id, and time
+  order = seq(0, n-1);
+  std::sort(order.begin(), order.end(), [&](int i, int j) {
+    return ((treatn[i] < treatn[j]) ||
+            ((treatn[i] == treatn[j]) && (idn[i] < idn[j])) ||
+            ((treatn[i] == treatn[j]) && (idn[i] == idn[j]) && 
+            (tstopn[i] < tstopn[j])));
+  });
   
   idn = idn[order];
   stratumn = stratumn[order];
@@ -544,7 +566,6 @@ List ipcwcpp(
                 IntegerVector& swtrtb, NumericVector& swtrt_timeb,
                 NumericMatrix& zb, NumericMatrix& zb_cox_den,
                 NumericMatrix& zb_lgs_den)->List {
-                  int h, i, j;
                   bool fail = 0; // whether any model fails to converge
                   NumericVector init(1, NA_REAL);
                   
@@ -570,7 +591,7 @@ List ipcwcpp(
                   
                   // set up crossover indicators
                   IntegerVector cross1(n1);
-                  for (i=0; i<n1; i++) {
+                  for (int i=0; i<n1; i++) {
                     if (i == n1-1 || id1[i] != id1[i+1]) {
                       if (swtrt1[i] == 1 && tstop1[i] >= swtrt_time1[i]) {
                         cross1[i] = 1;
@@ -581,7 +602,7 @@ List ipcwcpp(
                   // initialize data_switch and fit_switch
                   List data_switch(2), fit_switch(2);
                   if (k == -1) {
-                    for (h=0; h<2; h++) {
+                    for (int h=0; h<2; h++) {
                       List data_x = List::create(
                         Named("data") = R_NilValue,
                         Named(treat) = R_NilValue
@@ -653,7 +674,7 @@ List ipcwcpp(
                       z2 = subset_matrix_by_row(z1, l);
                       z2_cox_den = subset_matrix_by_row(z1_cox_den, l);
                       n2 = static_cast<int>(l.size());
-                      for (i=0; i<n2; i++) {
+                      for (int i=0; i<n2; i++) {
                         if (censor[i] == 1) {
                           event2[i] = 0;
                           cross2[i] = 0;
@@ -688,7 +709,7 @@ List ipcwcpp(
                       NumericMatrix z20_cox_den = 
                         subset_matrix_by_row(z10_cox_den, l);
                       int n20 = static_cast<int>(l.size());
-                      for (i=0; i<n20; i++) {
+                      for (int i=0; i<n20; i++) {
                         if (censor[i] == 1) {
                           event20[i] = 0;
                           cross20[i] = 0;
@@ -726,7 +747,7 @@ List ipcwcpp(
                     NumericVector w2(n2, NA_REAL), sw2(n2, NA_REAL);
                     
                     // fit the switching models by treatment group
-                    for (h=0; h<K; h++) {
+                    for (int h=0; h<K; h++) {
                       IntegerVector l = which(treat2 == h);
                       IntegerVector id3 = id2[l];
                       IntegerVector stratum3 = stratum2[l];
@@ -744,7 +765,7 @@ List ipcwcpp(
                         Named("tstop") = tstop3,
                         Named("cross") = cross3);
                       
-                      for (j=0; j<p2; j++) {
+                      for (int j=0; j<p2; j++) {
                         String zj = denominator[j];
                         NumericVector u = z3_cox_den(_,j);
                         data1.push_back(u,zj);
@@ -772,61 +793,50 @@ List ipcwcpp(
                         "tstop", 0, "log-log", 1-alpha);
                       
                       NumericVector surv_den = km_den["surv"];
-                      int m = km_den.nrows();
                       
                       List fit_num = phregcpp(
                         data1, "", "ustratum", "tstart", "tstop", "cross", 
                         numerator, "", "", "uid", ties, init, 
                         0, 1, 0, 0, 0, alpha, 50, 1.0e-9);
                       
-                      NumericVector surv_num(m);
+                      int p10 = p1 > 0 ? p1 : 1;
+                      NumericVector beta_num(p10);
+                      NumericMatrix vbeta_num(p10,p10);
                       if (p1 > 0) {
-                        DataFrame sumstat_num = DataFrame(fit_num["sumstat"]);
+                        DataFrame sumstat_num= DataFrame(fit_num["sumstat"]);
                         bool fail_num = sumstat_num["fail"];
                         if (fail_num == 1) fail = 1;
                         
                         DataFrame parest_num = DataFrame(fit_num["parest"]);
-                        NumericVector beta_num = parest_num["beta"];
-                        NumericMatrix vbeta_num(p1, p1);
-                        DataFrame basehaz_num = 
-                          DataFrame(fit_num["basehaz"]);
-                        
-                        DataFrame km_num = survfit_phregcpp(
-                          p1, beta_num, vbeta_num, basehaz_num, data1,
-                          numerator, "ustratum", "", "uid", "tstart", 
-                          "tstop", 0, "log-log", 1-alpha);
-                        
-                        surv_num = km_num["surv"];
-                      } else {
-                        NumericVector beta_num(1);
-                        NumericMatrix vbeta_num(1,1);
-                        DataFrame basehaz_num = 
-                          DataFrame(fit_num["basehaz"]);
-                        
-                        DataFrame km_num = survfit_phregcpp(
-                          p1, beta_num, vbeta_num, basehaz_num, data1,
-                          numerator, "ustratum", "", "uid", "tstart", 
-                          "tstop", 0, "log-log", 1-alpha);
-                        
-                        surv_num = km_num["surv"];
+                        beta_num = parest_num["beta"];
                       }
+                      DataFrame basehaz_num = DataFrame(fit_num["basehaz"]);
+                      
+                      DataFrame km_num = survfit_phregcpp(
+                        p1, beta_num, vbeta_num, basehaz_num, data1,
+                        numerator, "ustratum", "", "uid", "tstart", 
+                        "tstop", 0, "log-log", 1-alpha);
+                      
+                      NumericVector surv_num = km_num["surv"];
                       
                       // unstabilized and stabilized weights
                       NumericVector w = 1.0/surv_den;
                       NumericVector sw = surv_num/surv_den;
-                      
+
                       // truncate the weights if requested
                       if (trunc > 0.0) {
+                        int m = w.size();
+                        
                         // truncated unstabilized weights
                         if (trunc_upper_only) {
                           double upper = quantilecpp(w, 1-trunc);
-                          for (i=0; i<m; i++) {
+                          for (int i=0; i<m; i++) {
                             if (w[i] > upper) w[i] = upper;
                           }
                         } else {
                           double lower = quantilecpp(w, trunc);
                           double upper = quantilecpp(w, 1-trunc);
-                          for (i=0; i<m; i++) {
+                          for (int i=0; i<m; i++) {
                             if (w[i] < lower) {
                               w[i] = lower;
                             } else if (w[i] > upper) {
@@ -838,13 +848,13 @@ List ipcwcpp(
                         // truncated stabilized weights
                         if (trunc_upper_only) {
                           double upper = quantilecpp(sw, 1-trunc);
-                          for (i=0; i<m; i++) {
+                          for (int i=0; i<m; i++) {
                             if (sw[i] > upper) sw[i] = upper;
                           }
                         } else {
                           double lower = quantilecpp(sw, trunc);
                           double upper = quantilecpp(sw, 1-trunc);
-                          for (i=0; i<m; i++) {
+                          for (int i=0; i<m; i++) {
                             if (sw[i] < lower) {
                               sw[i] = lower;
                             } else if (sw[i] > upper) {
@@ -853,7 +863,7 @@ List ipcwcpp(
                           }
                         }
                       }
-                      
+
                       // update data_switch and fit_switch
                       if (k == -1) {
                         IntegerVector uid = data1["uid"];
@@ -867,7 +877,7 @@ List ipcwcpp(
                         
                         if (has_stratum) {
                           IntegerVector ustratum = data1["ustratum"];
-                          for (i=0; i<p_stratum; i++) {
+                          for (int i=0; i<p_stratum; i++) {
                             String s = stratum[i];
                             if (TYPEOF(data[s]) == INTSXP) {
                               IntegerVector stratumwi = u_stratum[s];
@@ -901,10 +911,10 @@ List ipcwcpp(
                       w2[l] = w3;
                       sw2[l] = sw3;
                     }
-                    
+
                     // fill in missing weights with LOCF starting at 1
                     IntegerVector idx(1,0);
-                    for (i=1; i<n2; i++) {
+                    for (int i=1; i<n2; i++) {
                       if (id2[i] != id2[i-1]) {
                         idx.push_back(i);
                       }
@@ -913,12 +923,12 @@ List ipcwcpp(
                     int nids2 = static_cast<int>(idx.size());
                     idx.push_back(n2);
                     
-                    for (i=0; i<nids2; i++) {
+                    for (int i=0; i<nids2; i++) {
                       if (std::isnan(w2[idx[i]])) {
                         w2[idx[i]] = 1.0;
                         sw2[idx[i]] = 1.0;
                       }
-                      for (j=idx[i]+1; j<idx[i+1]; j++) {
+                      for (int j=idx[i]+1; j<idx[i+1]; j++) {
                         if (std::isnan(w2[j])) {
                           w2[j] = w2[j-1];
                           sw2[j] = sw2[j-1];
@@ -938,7 +948,7 @@ List ipcwcpp(
                     
                     data_outcome.push_back(stratum2, "ustratum");
                     
-                    for (j=0; j<p; j++) {
+                    for (int j=0; j<p; j++) {
                       NumericVector u = z2(_,j);
                       String zj = base_cov[j];
                       data_outcome.push_back(u,zj);
@@ -947,9 +957,9 @@ List ipcwcpp(
                     NumericVector w1(n1, 1.0), sw1(n1, 1.0);
                     
                     // fit the switching models by treatment group
-                    for (h=0; h<K; h++) {
+                    for (int h=0; h<K; h++) {
                       LogicalVector c2 = ifelse(
-                        swtrt1 == 1, tstart1 < swtrt_time1, tstop1 < os_time1);
+                        swtrt1 == 1, tstart1 < swtrt_time1, tstop1<os_time1);
                       IntegerVector l = which(c2 & (treat1 == h));
                       IntegerVector id2 = id1[l];
                       IntegerVector stratum2 = stratum1[l];
@@ -981,12 +991,12 @@ List ipcwcpp(
                         Named("tstop") = tstop2,
                         Named("cross") = cross2);
                       
-                      for (j=0; j<q+p2; j++) {
+                      for (int j=0; j<q+p2; j++) {
                         String zj = covariates_lgs_den[j];
                         NumericVector u = z2_lgs_den(_,j);
                         data1.push_back(u,zj);
                       }
-                      for (j=0; j<ns_df; j++) {
+                      for (int j=0; j<ns_df; j++) {
                         String zj = covariates_lgs_den[q+p2+j];
                         NumericVector u = s(_,j);
                         data1.push_back(u,zj);
@@ -1018,7 +1028,7 @@ List ipcwcpp(
                       
                       // convert to probability of observed response 
                       NumericVector o_den(n2), o_num(n2);
-                      for (i=0; i<n2; i++) {
+                      for (int i=0; i<n2; i++) {
                         o_den[i] = cross2[i] == 1 ? h_den[i] : 1 - h_den[i];
                         o_num[i] = cross2[i] == 1 ? h_num[i] : 1 - h_num[i];
                       }
@@ -1030,7 +1040,7 @@ List ipcwcpp(
                       int n3 = static_cast<int>(l.size());
                       
                       IntegerVector idx3(1,0);
-                      for (i=1; i<n3; i++) {
+                      for (int i=1; i<n3; i++) {
                         if (id3[i] != id3[i-1]) {
                           idx3.push_back(i);
                         }
@@ -1044,9 +1054,9 @@ List ipcwcpp(
                       NumericVector p_den(n3, 1.0), p_num(n3, 1.0);
                       
                       int m = 0;
-                      for (i=0; i<nids3; i++) {
+                      for (int i=0; i<nids3; i++) {
                         int r = m - idx3[i] - 1;
-                        for (j=idx3[i]+1; j<idx3[i+1]; j++) {
+                        for (int j=idx3[i]+1; j<idx3[i+1]; j++) {
                           p_den[j] = p_den[j-1]*o_den[r+j];
                           p_num[j] = p_num[j-1]*o_num[r+j];
                         }
@@ -1065,13 +1075,13 @@ List ipcwcpp(
                         // truncated unstabilized weights
                         if (trunc_upper_only) {
                           double upper = quantilecpp(w, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (int i=0; i<n3; i++) {
                             if (w[i] > upper) w[i] = upper;
                           }
                         } else {
                           double lower = quantilecpp(w, trunc);
                           double upper = quantilecpp(w, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (int i=0; i<n3; i++) {
                             if (w[i] < lower) {
                               w[i] = lower;
                             } else if (w[i] > upper) {
@@ -1083,13 +1093,13 @@ List ipcwcpp(
                         // truncated stabilized weights
                         if (trunc_upper_only) {
                           double upper = quantilecpp(sw, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (int i=0; i<n3; i++) {
                             if (sw[i] > upper) sw[i] = upper;
                           }
                         } else {
                           double lower = quantilecpp(sw, trunc);
                           double upper = quantilecpp(sw, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (int i=0; i<n3; i++) {
                             if (sw[i] < lower) {
                               sw[i] = lower;
                             } else if (sw[i] > upper) {
@@ -1112,7 +1122,7 @@ List ipcwcpp(
                         
                         if (has_stratum) {
                           IntegerVector ustratum = data1["ustratum"];
-                          for (i=0; i<p_stratum; i++) {
+                          for (int i=0; i<p_stratum; i++) {
                             String s = stratum[i];
                             if (TYPEOF(data[s]) == INTSXP) {
                               IntegerVector stratumwi = u_stratum[s];
@@ -1153,7 +1163,7 @@ List ipcwcpp(
                     
                     data_outcome.push_back(stratum1, "ustratum");
                     
-                    for (j=0; j<p; j++) {
+                    for (int j=0; j<p; j++) {
                       NumericVector u = z1(_,j);
                       String zj = base_cov[j];
                       data_outcome.push_back(u,zj);
@@ -1186,7 +1196,7 @@ List ipcwcpp(
                   double hrlower = exp(beta[0] - zcrit*sebeta[0]);
                   double hrupper = exp(beta[0] + zcrit*sebeta[0]);
                   double pvalue = pval[0];
-                  
+
                   List out;
                   if (k == -1) {
                     out = List::create(
@@ -1283,6 +1293,31 @@ List ipcwcpp(
     NumericMatrix zc_lgs_den(B, q+p2);
     int index1 = 0;
     
+    if (has_stratum) {
+      // sort data by treatment group, stratum, id, and time
+      IntegerVector order = seq(0, n-1);
+      std::sort(order.begin(), order.end(), [&](int i, int j) {
+        return ((treatn[i] < treatn[j]) ||
+                ((treatn[i] == treatn[j]) && (stratumn[i] < stratumn[j])) ||
+                ((treatn[i] == treatn[j]) && (stratumn[i] == stratumn[j]) &&
+                (idn[i] < idn[j])) ||
+                ((treatn[i] == treatn[j]) && (stratumn[i] == stratumn[j]) &&
+                (idn[i] == idn[j]) && (tstopn[i] < tstopn[j])));
+      });
+      
+      idn = idn[order];
+      stratumn = stratumn[order];
+      tstartn = tstartn[order];
+      tstopn = tstopn[order];
+      eventn = eventn[order];
+      treatn = treatn[order];
+      swtrtn = swtrtn[order];
+      swtrt_timen = swtrt_timen[order];
+      zn = subset_matrix_by_row(zn, order);
+      zn_cox_den = subset_matrix_by_row(zn_cox_den, order);
+      zn_lgs_den = subset_matrix_by_row(zn_lgs_den, order);
+    }
+    
     IntegerVector tsx(1,0); // first id within each treat/stratum
     for (i=1; i<nids; i++) {
       if ((treatn1[i] != treatn1[i-1]) || 
