@@ -51,9 +51,13 @@ double est_psi_rpsftm(
       covariates, "", "", "", ties, init, 
       0, 0, 0, 0, 0, alpha, 50, 1.0e-9);
     
-    DataFrame parest = DataFrame(fit["parest"]);
-    NumericVector zs = parest["z"];
-    z = zs[0];
+    DataFrame sumstat = DataFrame(fit["sumstat"]);
+    bool fail = sumstat["fail"];
+    if (!fail) {
+      DataFrame parest = DataFrame(fit["parest"]);
+      NumericVector zs = parest["z"];
+      z = zs[0];
+    }
   } else if (test == "lifereg") {
     for (int j=0; j<q+p; j++) {
       String zj = covariates_aft[j+1];
@@ -65,9 +69,13 @@ double est_psi_rpsftm(
                           covariates_aft, "", "", "", dist, init, 
                           0, 0, alpha, 50, 1.0e-9);
     
-    DataFrame parest = DataFrame(fit["parest"]);
-    NumericVector zs = parest["z"];
-    z = -zs[1];
+    DataFrame sumstat = DataFrame(fit["sumstat"]);
+    bool fail = sumstat["fail"];
+    if (!fail) {
+      DataFrame parest = DataFrame(fit["parest"]);
+      NumericVector zs = parest["z"];
+      z = -zs[1];
+    }
   }
   
   return z;
@@ -95,6 +103,7 @@ List rpsftmcpp(const DataFrame data,
                const bool admin_recensor_only = 1,
                const bool autoswitch = 1,
                const bool gridsearch = 0,
+               const std::string root_finding = "brent",
                const double alpha = 0.05,
                const std::string ties = "efron",
                const double tol = 1.0e-6,
@@ -433,6 +442,21 @@ List rpsftmcpp(const DataFrame data,
     stop("treat_modifier must be positive");
   }
   
+  std::string rooting = root_finding;
+  std::for_each(rooting.begin(), rooting.end(), [](char & c) {
+    c = static_cast<char>(std::tolower(static_cast<unsigned char>(c)));
+  });
+  
+  if (rooting == "uniroot" || rooting.rfind("br", 0) == 0) {
+    rooting = "brent";
+  } else if (rooting.rfind("bi", 0) == 0) {
+    rooting = "bisect";
+  }
+  
+  if (!((rooting == "brent") || (rooting == "bisect"))) {
+    stop("root_finding must be brent or bisection");
+  }
+  
   if (alpha <= 0.0 || alpha >= 0.5) {
     stop("alpha must lie between 0 and 0.5");
   }
@@ -450,6 +474,34 @@ List rpsftmcpp(const DataFrame data,
   }
   
   
+  // exclude observations with missing values
+  LogicalVector sub(n,1);
+  for (i=0; i<n; i++) {
+    if ((idn[i] == NA_INTEGER) || (stratumn[i] == NA_INTEGER) || 
+        (std::isnan(timen[i])) || (eventn[i] == NA_INTEGER) || 
+        (treatn[i] == NA_INTEGER) || (std::isnan(rxn[i])) ||
+        (std::isnan(censor_timen[i]))) {
+      sub[i] = 0;
+    }
+    for (j=0; j<p; j++) {
+      if (std::isnan(zn_aft(i,j))) sub[i] = 0;
+    }
+  }
+  
+  IntegerVector order = which(sub);
+  idn = idn[order];
+  stratumn = stratumn[order];
+  timen = timen[order];
+  eventn = eventn[order];
+  treatn = treatn[order];
+  rxn = rxn[order];
+  censor_timen = censor_timen[order];
+  if (p > 0) zn = subset_matrix_by_row(zn, order);
+  zn_aft = subset_matrix_by_row(zn_aft, order);
+  n = sum(sub);
+  if (n == 0) stop("no observations left after removing missing values");
+  
+  
   DataFrame lr = lrtest(data, "", stratum, treat, time, event, 0, 0);
   double logRankPValue = lr["logRankPValue"];
   
@@ -473,13 +525,12 @@ List rpsftmcpp(const DataFrame data,
   k = -1;
   auto f = [&k, n, q, p, test, covariates, covariates_aft, dist, low_psi, 
             hi_psi, n_eval_z, psi, treat_modifier, recensor, autoswitch, 
-            gridsearch, alpha, zcrit, ties, tol](
+            gridsearch, rooting, alpha, zcrit, ties, tol](
                 IntegerVector& idb, 
                 IntegerVector& stratumb, NumericVector& timeb,
                 IntegerVector& eventb, IntegerVector& treatb,
                 NumericVector& rxb, NumericVector& censor_timeb,
                 NumericMatrix& zb, NumericMatrix& zb_aft)->List {
-                  int i, j;
                   bool fail = 0; // whether any model fails to converge
                   NumericVector init(1, NA_REAL);
                   
@@ -490,7 +541,7 @@ List rpsftmcpp(const DataFrame data,
                   
                   if (gridsearch) {
                     NumericVector Z(n_eval_z);
-                    for (i=0; i<n_eval_z; i++) {
+                    for (int i=0; i<n_eval_z; i++) {
                       Z[i] = est_psi_rpsftm(
                         psi[i], q, p, idb, stratumb, timeb, eventb, 
                         treatb, rxb, censor_timeb, test, covariates, zb, 
@@ -524,7 +575,11 @@ List rpsftmcpp(const DataFrame data,
                     double psilo = getpsiend(g, 1, low_psi);
                     double psihi = getpsiend(g, 0, hi_psi);
                     if (!std::isnan(psilo) && !std::isnan(psihi)) {
-                      psihat = brent(g, psilo, psihi, tol);
+                      if (rooting == "brent") {
+                        psihat = brent(g, psilo, psihi, tol);
+                      } else {
+                        psihat = bisect(g, psilo, psihi, tol);
+                      }
                     }
                     psi_CI_type = "root finding";
                     
@@ -533,10 +588,18 @@ List rpsftmcpp(const DataFrame data,
                       psilo = getpsiend(g, 1, low_psi);
                       psihi = getpsiend(g, 0, hi_psi);
                       if (!std::isnan(psilo) && !std::isnan(psihi)) {
-                        if (!std::isnan(psihat)) {
-                          psilower = brent(g, psilo, psihat, tol);
+                        if (!std::isnan(psihat) && g(psihat) < 0) {
+                          if (rooting == "brent") {
+                            psilower = brent(g, psilo, psihat, tol);
+                          } else {
+                            psilower = bisect(g, psilo, psihat, tol);
+                          }
                         } else {
-                          psilower = brent(g, psilo, psihi, tol);
+                          if (rooting == "brent") {
+                            psilower = brent(g, psilo, psihi, tol);
+                          } else {
+                            psilower = bisect(g, psilo, psihi, tol);
+                          }
                         }
                       }
                       
@@ -544,10 +607,18 @@ List rpsftmcpp(const DataFrame data,
                       psilo = getpsiend(g, 1, low_psi);
                       psihi = getpsiend(g, 0, hi_psi);
                       if (!std::isnan(psilo) && !std::isnan(psihi)) {
-                        if (!std::isnan(psihat)) {
-                          psiupper = brent(g, psihat, psihi, tol);  
+                        if (!std::isnan(psihat) && g(psihat) > 0) {
+                          if (rooting == "brent") {
+                            psiupper = brent(g, psihat, psihi, tol);  
+                          } else {
+                            psiupper = bisect(g, psihat, psihi, tol);  
+                          }
                         } else {
-                          psiupper = brent(g, psilo, psihi, tol);  
+                          if (rooting == "brent") {
+                            psiupper = brent(g, psilo, psihi, tol);  
+                          } else {
+                            psiupper = bisect(g, psilo, psihi, tol);
+                          }
                         }
                       }
                     }
@@ -571,7 +642,7 @@ List rpsftmcpp(const DataFrame data,
                       
                       Sstar.push_back(stratumb, "ustratum");
                       
-                      for (j=0; j<p; j++) {
+                      for (int j=0; j<p; j++) {
                         String zj = covariates[j+1];
                         NumericVector u = zb(_,j);
                         Sstar.push_back(u, zj);
@@ -585,7 +656,7 @@ List rpsftmcpp(const DataFrame data,
                     
                     data_outcome.push_back(stratumb, "ustratum");
                     
-                    for (j=0; j<p; j++) {
+                    for (int j=0; j<p; j++) {
                       String zj = covariates[j+1];
                       NumericVector u = zb(_,j);
                       data_outcome.push_back(u, zj);
@@ -930,6 +1001,7 @@ List rpsftmcpp(const DataFrame data,
     Named("admin_recensor_only") = admin_recensor_only,
     Named("autoswitch") = autoswitch,
     Named("gridsearch") = gridsearch,
+    Named("root_finding") = root_finding,
     Named("alpha") = alpha,
     Named("ties") = ties,
     Named("tol") = tol,
