@@ -10,11 +10,16 @@ void set_seed(int seed);
 
 IntegerVector which(const LogicalVector& vector);
 
-IntegerVector findInterval3(NumericVector x, NumericVector v);
+IntegerVector findInterval3(NumericVector x, NumericVector v, 
+                            bool rightmost_closed, bool all_inside, 
+                            bool left_open);
 
 double brent(const std::function<double(double)>& f,
              double x1, double x2, double tol);
 
+double bisect(const std::function<double(double)>& f,
+              double x1, double x2, double tol);
+
 bool hasVariable(DataFrame df, std::string varName);
 
 double quantilecpp(const NumericVector& x, const double p);
@@ -80,11 +85,6 @@ double qtpwexpcpp1(const double p,
                    const bool lowertail,
                    const bool logp);
 
-NumericVector getAccrualDurationFromN(
-    const NumericVector& nsubjects,
-    const NumericVector& accrualTime,
-    const NumericVector& accrualIntensity);
-
 double getpsiest(const double target, const NumericVector& psi, 
                  const NumericVector& Z);
 
